import time
from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.access import get_active_company

from . import utils


class ReportHDCaseDetail(Model):
    _name="clinic.report.hd.case.detail"
    _string="Report HDCase Detail"
    _transient=True
    
    _fields={
        "date": fields.Date("Month"),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        'branch_id': fields.Many2One("clinic.branch","Branch"),
        'department_id': fields.Many2One("clinic.department","Department"),
        'patient_type_id': fields.Many2One("clinic.patient.type","Patient Type"),
        'hdcase_type': fields.Selection([['completed','Completed'],['not_completed','Not Completed']],"HDCase Type"),
        'report_type': fields.Selection([
                ['topic1','จำนวนครั้งการทำ Hemodialysis'],
                ['topic2','จำนวนผู้ป่วยยกมาจากเดือน'],
                ['topic3','จำนวนผู้ป่วยรับใหม่เดือน'],
                ['topic4','จำนวนผู้ป่วยจำหน่ายเดือน'],
                ['topic5','จำนวนผู้ป่วยยกไปเดือน'],
                ['topic6','ประเภทผู้ป่วย'],
            ],"Repor Type",required=True),
    }

    def default_get(self,field_names=None,context={},**kw):
        defaults=context.get("defaults",{})
        date=defaults.get('date',time.strftime("%Y-%m-%d"))
        year,month=time.strftime("%Y-%m").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        date_from=defaults.get('date_from','%s-%s-01'%(year,month))
        date_to=defaults.get('date_to',"%s-%s-%s"%(year,month,total_day))
        report_type=defaults.get('report_type',"topic1")
        if report_type:
            index=''
            for r in report_type:
                if r.isdigit():
                    index+=r
            if int(index)>=6:
                report_type='topic6'
        branch_id=defaults.get('branch_id',None)
        hdcase_type=defaults.get("hdcase_type","completed")
        patient_type_id=defaults.get('patient_type_id')
        if patient_type_id:
            patient_type_id=int(patient_type_id)
        select_dpt=get_model('select.company').get_select()
        if not branch_id:
            if select_dpt:
                branch_id=select_dpt['branch_id']
        else:
            branch_id=int(branch_id or "0")
        department_id=defaults.get('department_id',None)
        if not department_id:
            if select_dpt.get('department_ids'):
                department_id=select_dpt['department_ids'][0]
            else:
                department_id=select_dpt['department_id']
        else:
            department_id=int(department_id)
        res={
            'date': date,
            'date_from': date_from,
            'date_to': date_to,
            'branch_id': branch_id,
            'department_id': department_id,
            'report_type': report_type,
            'hdcase_type': hdcase_type,
            'patient_type_id': patient_type_id,
        }
        #print('report.hd.case.detail.defaults ', res)
        return res

    def get_report_data(self,ids,context={}):
        company_id=get_active_company()
        company=get_model("company").browse(company_id)
        defaults=self.default_get(context=context)
        date=defaults.get('date',time.strftime("%Y-%m-%d"))
        year=int(date[0:4])
        crr_month=int(date[5:7])
        weekday, crr_total_day=monthrange(year, crr_month)
        date_from=defaults.get('date_from',date)
        date_to=defaults.get('date_to',date)
        branch_id=defaults.get("branch_id",None)
        department_id=defaults.get("department_id",None)
        hdcase_type=defaults.get('hdcase_type')
        report_type=defaults.get('report_type','topic1')
        patient_type_id=defaults.get('patient_type_id')
        time_start='%s-%s-01'%(year,str(crr_month).zfill(2))
        time_stop='%s-%s-%s'%(year,str(crr_month).zfill(2),crr_total_day)
        titles=dict(self._fields['report_type'].selection)
        title=titles.get(report_type,'no title')
        if ids:
            obj=self.browse(ids)[0]
            branch_id=obj.branch_id.id
            department_id=obj.department_id.id
            date=obj.date 
            crr_month=int(date[5:7])
            time_start=obj.date_from
            time_stop=obj.date_to
            year=int(date[0:4])
            date_from=obj.date_from
            date_to=obj.date_to
            report_type=obj.report_type or 'topic1'
            hdcase_type=obj.hdcase_type or 'completed'
            patient_type_id=obj.patient_type_id.id
        prev_year=year
        next_year=year
        prev_month=crr_month-1
        next_month=crr_month+1
        if crr_month==1:
            prev_month=12
            prev_year-=1
        if crr_month==12:
            next_month=1
            next_year+=1
        month_str=utils.MONTHS['th_TH'][crr_month]
        next_month_str=utils.MONTHS['th_TH'][next_month]
        prev_month_str=utils.MONTHS['th_TH'][prev_month]
              
        def replace_quote(dom=""):
            dom=dom.replace("False","false")
            dom=dom.replace("True","true")
            return dom.replace("'","\"")
        dom=[]
        lines=[]
        total=0
        # current hd case
        if report_type=='topic1':
            title=titles.get(report_type,'no title')
            dom=[]
            dom.append(["date",">=",date_from])
            dom.append(["date","<=",date_to])
            if hdcase_type=='completed':
                dom.append(["state","in",["waiting_payment","paid"]])
            else:
                dom.append(["state","not in",["waiting_payment","paid"]])
            if branch_id:
                dom.append(['branch_id','=',branch_id])
            if department_id:
                dom.append(['department_id','=',department_id])
            total=0
            for hdcase in get_model("clinic.hd.case").search_browse(dom,order="number"):
                patient=hdcase.patient_id
                lines.append({
                    'number': hdcase.number,
                    'date': hdcase.date,
                    'name': patient.name,
                    'note': hdcase.note,
                })
                total+=1 
        # number of patient from previous month
        elif report_type=='topic2':
            title=titles.get(report_type,'no title')
            title+=' '+prev_month_str
            prev_weekday, prev_total_day=monthrange(prev_year, prev_month)
            reg_date='%s-%s-%s'%(prev_year,str(prev_month).zfill(2),str(prev_total_day).zfill(2))
            print('topic2 ', reg_date)
            dom=[]
            dom.append(['reg_date','<=',reg_date])
            dom.append(['walkin','=',"no"])
            #dom.append(['dispose','=',False])
            if branch_id:
                dom.append(['branch_id','=',branch_id])
            if department_id:
                dom.append(['department_id','=',department_id])
            total=0
            for patient in get_model("clinic.patient").search_browse(dom):
                if patient.dispose and patient.resign_date:
                    if patient.resign_date < date_from:
                        print('continue >>> ', patient.name, patient.resign_date, date_from, date_to)
                        continue
                    elif patient.resign_date > date_to:
                        pass
                    elif not patient.resign_date > date_to:
                        print('pass >>> ', patient.name, patient.resign_date, date_from, date_to)
                    elif not (patient.resign_date>=date_from and patient.resign_date<=date_to):
                        print('continue >>> ', patient.name, patient.resign_date, date_from, date_to)
                        continue
                    print('pass >>> ', patient.name, patient.resign_date, date_from, date_to)
                lines.append({
                    'number': patient.hn_no,
                    'pid': patient.id,
                    'name': patient.name,
                    'note': patient.note,
                    'date': patient.reg_date,
                })
                total+=1
        # new patient of this month
        #TODO should include move_history
        elif report_type=='topic3':
            title=titles.get(report_type,'no title')
            title+=' '+month_str
            dom=[]
            dom.append(['reg_date','>=',time_start])
            dom.append(['reg_date','<=',time_stop])
            dom.append(['walkin','=',"no"])
            #dom.append(['dispose','=',False])
            if branch_id:
                dom.append(['branch_id','=',branch_id])
            if department_id:
                dom.append(['department_id','=',department_id])
            total=0
            for patient in get_model('clinic.patient').search_browse(dom):
                lines.append({
                    'number': patient.hn_no,
                    'pid': patient.id,
                    'name': patient.name,
                    'note': patient.note,
                    'date': patient.reg_date,
                })
                total+=1
        # number for patient who resign for this month
        #TODO should include move_history
        elif report_type=='topic4':
            title=titles.get(report_type,'no title')
            title+=' '+month_str
            dom=[]
            dom.append(['resign_date','>=',time_start])
            dom.append(['resign_date','<=',time_stop])
            dom.append(['walkin','=',"no"])
            dom.append(['dispose','=',True])
            if branch_id:
                dom.append(['branch_id','=',branch_id])
            if department_id:
                dom.append(['department_id','=',department_id])
            total=0
            for patient in get_model('clinic.patient').search_browse(dom):
                lines.append({
                    'number': patient.hn_no,
                    'pid': patient.id,
                    'name': patient.name,
                    'note': patient.note,
                    'date': patient.resign_date,
                })
                total+=1
        # all patient who are in hospital on select month
        elif report_type=='topic5':
            print('topic5 ', time_stop)
            title=titles.get(report_type,'no title')
            title+=' '+next_month_str
            dom=[]
            dom.append(['reg_date','<=',time_stop])
            if branch_id:
                dom.append(['branch_id','=',branch_id])
            if department_id:
                dom.append(['department_id','=',department_id])
            dom.append(['walkin','=',"no"])
            total=0
            for patient in get_model('clinic.patient').search_browse(dom,order="reg_date"):
                if patient.dispose and patient.resign_date:
                    if patient.resign_date < date_from:
                        print('1. continue >>> ', patient.name, patient.resign_date, date_from, date_to)
                        continue
                    elif patient.resign_date > date_to:
                        pass
                    elif patient.resign_date > date_to:
                        print('2. pass >>> ', patient.name, patient.resign_date, date_from, date_to)
                    elif (patient.resign_date>=date_from and patient.resign_date<=date_to):
                        print('3. continue >>> ', patient.name, patient.resign_date, date_from, date_to)
                        continue
                    print('4. pass >>> ', patient.name, patient.resign_date, date_from, date_to)
                lines.append({
                    'number': patient.hn_no,
                    'pid': patient.id,
                    'name': patient.name,
                    'note': patient.note,
                    'date': patient.reg_date,
                    'dispose': patient.dispose and 'yes' or 'no',
                })
                total+=1
        # find patient movement
        elif report_type=='topic6':
            print('topic6')
            ptype=None
            if patient_type_id:
                ptype=get_model('clinic.patient.type').browse(patient_type_id)
            if not ptype:
                return {'total': 0}
            title=ptype.name or ''
            dom=[]
            dom.append(['reg_date','<=',time_stop])
            dom.append(['type_id','=',ptype['id']])
            dom.append(['walkin','=',"no"])
            dom.append(['type_id','=',ptype.id])
            if branch_id:
                dom.append(['branch_id','=',branch_id])
            if department_id:
                dom.append(['department_id','=',department_id])
            total=0
            for patient in get_model("clinic.patient").search_browse(dom):
                if patient.resign_date:
                    if patient.dispose and patient.resign_date < date_from:
                        print('1. continue >>> ', patient.name, patient.resign_date, date_from, date_to)
                        continue
                    elif patient.resign_date > date_to:
                        pass
                    elif patient.resign_date > date_to:
                        print('2. pass >>> ', patient.name, patient.resign_date, date_from, date_to)
                    elif (patient.resign_date>=date_from and patient.resign_date<=date_to):
                        print('3. continue >>> ', patient.name, patient.resign_date, date_from, date_to)
                        continue
                    print('4. pass >>> ', patient.name, patient.resign_date, date_from, date_to)
                lines.append({
                    'pid': patient.id,
                    'number': patient.hn_no,
                    'name': patient.name or '',
                    'date': patient.reg_date,
                })
                total+=1
        sub_name=''
        if department_id:
            dpt=get_model("clinic.department").browse(department_id)
            sub_name="(%s)" % dpt.name or ""
        elif branch_id:
            branch=get_model("clinic.branch").browse(branch_id)
            sub_name="(%s)" % branch.name or ""
        no=1
        for line in lines:
            line['no']="{0:,.0f}".format(no)
            no+=1
        data={
            'total_txt': "{0:,.0f}".format(total),
            'total': total, 
            'title': title,
            'report_type': report_type,
            'date': date,
            'date_from': date_from,
            'date_to': date_to,
            'month': month_str,
            'year': year,
            'branch_id': branch_id,
            'department_id': department_id,
            'lines': lines,
            'company_name': '%s %s'% (company.name or "", sub_name),
            'parent_company_name': company.parent_id.name or "",
        }
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_date_from(self,context={}):
        data=context['data']
        data['date_to']=data['date_from']
        return data

ReportHDCaseDetail.register()
